@@ -30,7 +30,10 @@ from compliance.utils.data_parse import (
     format_json, get_sha256_hash, parse_dot_key
 )
 from compliance.utils.exceptions import (
-    EvidenceNotFoundError, HistoricalEvidenceNotFoundError, StaleEvidenceError
+    EvidenceNotFoundError,
+    HistoricalEvidenceNotFoundError,
+    LockerPushError,
+    StaleEvidenceError
 )
 
 import git
@@ -38,7 +41,8 @@ import git
 INDEX_FILE = 'index.json'
 DAY = 60 * 60 * 24
 AE_DEFAULT = 30 * DAY
-AE_EXEMPT = [INDEX_FILE, 'README.md', 'readme.md']
+READMES = ['README.md', 'readme.md', 'Readme.md']
+AE_EXEMPT = [INDEX_FILE] + READMES
 
 
 class Locker(object):
@@ -180,11 +184,10 @@ class Locker(object):
 
     def logger_init_msgs(self):
         """Log locker initialization information."""
-        self.logger.info(f'Local locker location is {self.local_path}')
         gpgsign = self.gitconfig.get('commit', {}).get('gpgsign')
         if self._do_push and not gpgsign:
             self.logger.warning(
-                'Commits will not be cryptographically signed, best '
+                'Commits may not be cryptographically signed, best '
                 'practice is to set gitconfig.commit.gpgsign to true '
                 'in your locker configuration.'
             )
@@ -409,24 +412,27 @@ class Locker(object):
         """
         return os.path.join(self.local_path, filename)
 
-    def get_remote_location(self, filename, include_commit=True):
+    def get_remote_location(self, filename, include_commit=True, sha=None):
         """
         Provide the path for a file/commit SHA in the locker.
 
         The file may or may not exist in the locker.
 
         :param filename: the name of a file in the locker.
-        :param include_commit: if the commit SHA should be included.
+        :param include_commit: if the latest commit SHA should be included.
+        :param sha: use this commit SHA; requires include_commit to be False.
 
         :returns: the remote repository path to the filename provided.
         """
         if not self.repo_url_with_creds:
             return os.path.join(self.local_path, filename)
 
-        ref = 'master'
+        ref = self.branch
         if include_commit:
             ref = self.repo.head.commit.hexsha
-        return f'{self.repo_url}/blob/{ref}/{filename}'
+        elif not include_commit and sha:
+            ref = sha
+        return f'{self.repo_url}/blob/{ref}/{filename.strip("/")}'
 
     def get_evidence(self, evidence_path, ignore_ttl=False, evidence_dt=None):
         """
@@ -523,8 +529,12 @@ class Locker(object):
     def checkout(self):
         """Pull (clone) the remote repository to the local git repository."""
         if os.path.isdir(os.path.join(self.local_path, '.git')):
+            self.logger.info(f'Using locker found in {self.local_path}...')
             self.repo = git.Repo(self.local_path)
         else:
+            self.logger.info(
+                f'Cloning locker {self.repo_url} to {self.local_path}...'
+            )
             self.repo = git.Repo.clone_from(
                 self.repo_url_with_creds, self.local_path, branch=self.branch
             )
@@ -550,6 +560,9 @@ class Locker(object):
                 f'Files updated at local time {time.ctime(time.time())}'
                 f'\n\n{updated_files}'
             )
+        self.logger.info(
+            f'Committing changes to local locker in {self.local_path}...'
+        )
         try:
             diff = self.repo.index.diff('HEAD')
             if len(diff) > 0:
@@ -560,7 +573,12 @@ class Locker(object):
     def push(self):
         """Push the local git repository to the remote repository."""
         if self._do_push:
-            self.repo.remotes[0].push()
+            self.logger.info(
+                f'Pushing local locker to remote repo {self.repo_url}...'
+            )
+            push_info = self.repo.remotes[0].push()[0]
+            if push_info.flags >= git.remote.PushInfo.ERROR:
+                raise LockerPushError(push_info)
 
     def add_content_to_locker(self, content, folder='', filename=None):
         """
@@ -580,6 +598,22 @@ class Locker(object):
             f.write(content)
         self.repo.index.add([results_file])
 
+    def get_content_from_locker(self, folder='', filename=None):
+        """
+        Read non-evidence related content from the local locker.
+
+        :param folder: the folder in the local locker to get the content from.
+        :param filename: the name of the file in the local locker.
+        """
+        if not filename:
+            raise ValueError('Filename cannot be empty.')
+        file_path = os.path.join(self.local_path, folder, filename)
+        content = None
+        if os.path.exists(file_path):
+            with open(file_path) as f:
+                content = f.read()
+        return content
+
     def get_reports_metadata(self):
         """
         Provide all metadata related to reports as a dictionary.
@@ -694,7 +728,7 @@ class Locker(object):
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         """
-        Handle local locker checkin and remote push if applicable.
+        Handle local locker check-in and remote push if applicable.
 
         Log an exception if raised, commit the files to the repo and if
         configured push it up to the `repo_url`.
@@ -708,8 +742,10 @@ class Locker(object):
 
     def _repo_init(self):
         if os.path.isdir(os.path.join(self.local_path, '.git')):
+            self.logger.info(f'Using locker found in {self.local_path}...')
             self.repo = git.Repo(self.local_path)
         else:
+            self.logger.info(f'Creating locker in {self.local_path}...')
             self.repo = git.Repo.init(self.local_path)
         self.init_config()
 
